/// <reference types="vinyl" />
/// <reference types="node" />
import { GulpTransformBase, File, GulpTransformBaseOptions, TransformCallback } from "gulptransformbase";
export declare function syntaxHighlighter(options?: SyntaxHighlighterOptions): SyntaxHighlighterTransform;
export interface ToggleConfigMessage {
    when?: "Always" | "Hidden" | "Never";
    prefix?: string;
    prefixShow?: string;
    prefixHide?: string;
    useTitle?: boolean;
    hideMessage?: string;
    showMessage?: string;
    message?: string;
    placement?: "Right" | "Below";
}
export interface ClassNames {
    toggleContainer?: string;
    toggleText?: string;
    toggle?: string;
    showToggle?: string;
    hideToggle?: string;
    isShowing?: string;
}
export interface ToggleConfig {
    toggleState?: "Show" | "Hide";
    message?: ToggleConfigMessage;
    createToggleFn?: string;
    classNames?: ClassNames;
    customCss?: string;
}
export interface SyntaxHighlighterOptions {
    isPartialFn?: (html: string, file: File) => boolean;
    useMinifiedSyntaxHighlighter?: boolean;
    minifiedOutput?: boolean;
    theme?: string;
    customTheme?: string;
    additionalCss?: string;
    globalParams?: {
        /** Additional CSS class names to be added to highlighter elements. */
        /** default '' */
        "class-name"?: string;
        /** First line number. */
        /** Default 1 */
        "first-line"?: number;
        /**
         * Pads line numbers. Possible values are:
         *
         *   false - don't pad line numbers. ( default )
         *   true  - automaticaly pad numbers with minimum required number of leading zeroes.
         *   [int] - length up to which pad line numbers.
         */
        'pad-line-numbers'?: boolean;
        /** Lines to highlight. */
        'highlight'?: number[];
        /** Title to be displayed above the code block. */
        'title'?: string;
        /** Enables or disables smart tabs. */
        /** default true */
        'smart-tabs': boolean;
        /** Gets or sets tab size. */
        /** Default 4 */
        'tab-size': number;
        /** Enables or disables gutter. */
        /** default true */
        'gutter': boolean;
        /** Enables or disables toolbar. */
        /** Default true */
        /** Enables quick code copy and paste from double click. */
        /** default true */
        /** Forces code view to be collapsed. */
        /** Enables or disables automatic links. */
        /** default true */
        'auto-links': boolean;
        /** Gets or sets light mode. Equavalent to turning off gutter and toolbar. */
        /** default false */
        /** default true */
        'unindent': boolean;
        'html-script': false;
    };
    config?: {
        space?: string;
        /** Enables use of <SCRIPT type="syntaxhighlighter" /> tags. */
        /** default true */
        useScriptTags?: boolean;
        /** Blogger mode flag. */
        /** default false */
        bloggerMode?: boolean;
        stripBrs?: boolean;
        /** Name of the tag that SyntaxHighlighter will automatically look for. */
        /** default pre */
        tagName?: string;
    };
    toggleConfig?: ToggleConfig;
}
export interface SyntaxHighlighterTransformOptions extends GulpTransformBaseOptions, SyntaxHighlighterOptions {
}
export declare class SyntaxHighlighterTransform extends GulpTransformBase<SyntaxHighlighterTransformOptions> {
    private removableScriptClassName;
    private globalParams;
    private useMinifiedSyntaxHighlighter;
    private minifiedOutput;
    private file;
    private document;
    private dom;
    private html;
    constructor(options: SyntaxHighlighterOptions);
    private applySyntaxHighlighter();
    private addRemovableScriptElement(contents);
    private getShCore();
    private getBrushFiles();
    private loadSyntaxHighlighterScripts();
    private setUpDocument(html);
    protected transformBufferFile(file: File, contents: Buffer, encoding: string, cb: TransformCallback): void;
    private addToggleDefaults();
    private addDefaultsToMessageConfig(toggleConfig);
    private addDefaultClassNames(toggleConfig);
    private defaultCreateToggleFn();
    private readToggle();
    private getToggleScript();
    private addToggleJs();
    private getDefaultToggleCss(classNames);
    private addToggleCss(minifiedOutput, classNames, customCss?);
    private addToggle();
    private addCss();
    private getTheme();
    private addTheme();
    private addAdditionalCss();
    private getPossiblyMinifiedCss(css);
    private addPossiblyMinifiedCss(css);
    private addCssToDocument(css);
    private getDocTypeString();
    private isPartial;
    private getNewContents();
    private getHtmlWithoutInjectedScripts();
    private getBodyWithoutInjectedScripts(outerHTML);
}
