"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
function removeClass(target, className) {
    target.className = target.className.replace(className, '');
}
;
function addClass(target, className) {
    target.className = target.className + " " + className;
}
;
function addClassToSVG(target, className) {
    var cls = target.getAttribute("class");
    if (cls) {
        cls += " " + className;
    }
    else {
        cls = className;
    }
    target.setAttribute("class", cls);
}
function getCaption(highlighterElement) {
    var caption = highlighterElement.children[0].caption;
    if (caption) {
        var text = caption.textContent;
        return text !== null ? text : "";
    }
    return "";
}
function addClickHandler(el, handler) {
    if (el.addEventListener) {
        el.addEventListener('click', handler, false);
    }
    else if (el.attachEvent) {
        el.attachEvent('onclick', handler);
    }
}
function createToggle(highlighterElement, hide, messages, when, config) {
    var isHidden = false;
    function svgClick() {
        highlighterElement.parentNode.removeChild(currentEl);
        addToggle(!isHidden);
    }
    function createToggleAndHandleClick(show) {
        var toggle = createToggleElement(show);
        addClickHandler(toggle, svgClick);
        addClassToSVG(toggle, config.classNames.toggle);
        addClassToSVG(toggle, show ? config.classNames.showToggle : config.classNames.hideToggle);
        return toggle;
    }
    var showElement = createToggleAndHandleClick(true);
    var hideElement = createToggleAndHandleClick(false);
    var currentEl;
    function addToggle(hidden) {
        if (currentEl && !isHidden) {
            removeClass(currentEl, config.classNames.isShowing);
        }
        var messageConfig = config.message;
        isHidden = hidden;
        currentEl = document.createElement("div");
        currentEl.className = config.classNames.toggleContainer;
        currentEl.appendChild(hidden ? showElement : hideElement);
        if (when === "Always" || when === "Hidden" && hidden) {
            var message = hidden ? messages.showMessage : messages.hiddenMessage;
            if (message) {
                var textEl = document.createElement(messageConfig.placement === "Below" ? "div" : "span");
                textEl.className = config.classNames.toggleText;
                textEl.innerHTML = message;
                currentEl.appendChild(textEl);
            }
        }
        highlighterElement.parentNode.insertBefore(currentEl, highlighterElement);
        if (hidden) {
            addClass(highlighterElement, "collapsed");
        }
        else {
            addClass(currentEl, config.classNames.isShowing);
            removeClass(highlighterElement, "collapsed");
        }
    }
    addToggle(hide);
}
var ToggleState;
(function (ToggleState) {
    ToggleState[ToggleState["Never"] = 0] = "Never";
    ToggleState[ToggleState["Show"] = 1] = "Show";
    ToggleState[ToggleState["Hide"] = 2] = "Hide";
})(ToggleState || (ToggleState = {}));
;
function setUpToggle(config) {
    function getToggleState(syntaxHighlighter) {
        var dataToggle = syntaxHighlighter.getAttribute("data-toggleState");
        var toggleState = ToggleState.Never;
        if (dataToggle === "Show") {
            toggleState = ToggleState.Show;
        }
        else if (dataToggle === "Hide") {
            toggleState = ToggleState.Hide;
        }
        else if (dataToggle === "Never") {
            toggleState = ToggleState.Never;
        }
        else {
            if (config.toggleState === "Show") {
                toggleState = ToggleState.Show;
            }
            else if (config.toggleState === "Hide") {
                toggleState = ToggleState.Hide;
            }
        }
        return toggleState;
    }
    function mergeMessages(syntaxHighlighter) {
        var messageConfig = __assign({}, config.message);
        var configNames = Object.getOwnPropertyNames(messageConfig);
        for (var i = 0; i < configNames.length; i++) {
            var propertyName = configNames[i];
            var dataValue = syntaxHighlighter.getAttribute("data-toggle" + propertyName);
            if (dataValue !== null) {
                messageConfig[propertyName] = dataValue;
            }
        }
        return messageConfig;
    }
    function getMessages(message, syntaxhighlighter) {
        var showMessage = "";
        var hideMessage = "";
        if (message.useTitle) {
            var caption = getCaption(syntaxHighlighter);
            showMessage = caption;
            hideMessage = caption;
        }
        else {
            showMessage = message.showMessage !== "" ? message.showMessage : message.message;
            hideMessage = message.hideMessage !== "" ? message.hideMessage : message.message;
        }
        showMessage = (message.prefixShow !== "" ? message.prefixShow : message.prefix) + showMessage;
        hideMessage = (message.prefixHide !== "" ? message.prefixHide : message.prefix) + hideMessage;
        return {
            hiddenMessage: hideMessage,
            showMessage: showMessage
        };
    }
    var syntaxHighlighters = window.document.getElementsByClassName("syntaxhighlighter");
    for (var i = 0; i < syntaxHighlighters.length; i++) {
        var syntaxHighlighter = syntaxHighlighters[i];
        var toggleState = getToggleState(syntaxHighlighter);
        if (toggleState !== ToggleState.Never) {
            var mergedMessages = mergeMessages(syntaxHighlighter);
            var show = toggleState === ToggleState.Show;
            createToggle(syntaxHighlighter, !show, getMessages(mergedMessages, syntaxHighlighter), mergedMessages.when, config);
        }
    }
}
